from finam.events.event import EventClient
from finam.order.order import OrderClient
from finam.portfolio.portfolio import PortfolioClient
from finam.securities.securities import SecurityClient


class Client:
    def __init__(self, token: str):
        self.portfolio = PortfolioClient(token)
        self.securities = SecurityClient(token)
        self.orders = OrderClient(token)
        self.event = EventClient(token)


if __name__ == "__main__":
    import asyncio

    from finam.order.model import (
        BoardType,
        Condition,
        ConditionType,
        CreateOrderRequestModel,
        DelOrderModel,
        OrdersRequestModel,
        OrderType,
        PropertyType,
        ValidateBefore,
        ValidBeforeType,
    )
    from finam.portfolio.model import PortfolioRequestModel

    client_id = "613395R9A27"
    token = "CAEQzRYaGAGgZoXjzsERyBBOommi+wP3ltLSIPXWBw=="
    code = "SiZ2"
    client = Client(token)

    print(asyncio.run(client.securities.get_data(code="Siz2")))

    # смотрим портфель
    # params = PortfolioRequestModel(clientId=client_id)
    # print(asyncio.run(client.portfolio.get_portfolio(params)))

    # выставляем заявку
    # payload = CreateOrderRequestModel(
    #     clientId=client_id,
    #     board="FUT",
    #     securityCode=code,
    #     buySell=OrderType.Buy,
    #     quantity=1,
    #     price=61111,
    #     property=PropertyType.PutInQueue,
    #     condition=None,
    #     validateBefore=None,
    # )
    # r = asyncio.run(client.orders.create_order(payload))
    # print(r)
    # transaction_id = r.data.transactionId
    # print(transaction_id)
    #
    # # # смотрим заявки
    # params = OrdersRequestModel(clientId=client_id)
    # print(asyncio.run(client.orders.get_orders(params)))
    #
    # # # выставляем СЛ
    # payload = CreateOrderRequestModel(
    #     clientId=client_id,
    #     board="FUT",
    #     securityCode=code,
    #     buySell=OrderType.Sell,
    #     quantity=1,
    #     price=61090,
    #     property=PropertyType.PutInQueue,
    #     condition=Condition(
    #         price=61090,
    #         type=ConditionType.AskOrLast
    #     ),
    #     validateBefore=ValidateBefore(
    #         type=ValidBeforeType.TillCancelled
    #     ),
    # )
    # r = asyncio.run(client.orders.create_order(payload))
    # print(r)
    # transaction_id = r.data.transactionId
    # print(transaction_id)
    #
    # # выставляем ТП
    # payload = CreateOrderRequestModel(
    #     clientId=client_id,
    #     board=BoardType.Futures,
    #     securityCode=code,
    #     buySell=OrderType.Sell,
    #     quantity=1,
    #     price=None,
    #     property=PropertyType.PutInQueue,
    #     condition=Condition(
    #         price=61150,
    #         type=ConditionType.LastUp
    #     ),
    #     validateBefore=ValidateBefore(
    #         type=ValidBeforeType.TillCancelled
    #     ),
    # )
    # r = asyncio.run(client.orders.create_order(payload))
    # print(r)
    # transaction_id = r.data.transactionId
    # print(transaction_id)
    #
    # # смотрим портфель
    # params = PortfolioRequestModel(clientId=client_id)
    # print(asyncio.run(client.portfolio.get_portfolio(params)))
    #
    # # смотрим заявки
    # params = OrdersRequestModel(clientId=client_id)
    # print(asyncio.run(client.orders.get_orders(params)))
    #
    # # смотрим заявки
    # params = OrdersRequestModel(clientId=client_id)
    # print(asyncio.run(client.orders.get_stop_orders(params)))

    # params = DelOrderModel(clientId=client_id, transactionId=transaction_id)
    # print(asyncio.run(client.orders.del_order(params)))



from pydantic import BaseModel


class PortfolioRequestOptionalModel(BaseModel):
    includeCurrencies: str = "true"
    includeMoney: str = "false"
    includeMaxBuySell: str = "true"
    includePositions: str = "true"


class PortfolioRequestModel(PortfolioRequestOptionalModel):
    clientId: str


class PortfolioContent(BaseModel):
    includeCurrencies: bool
    includeMoney: bool
    includePositions: bool
    includeMaxBuySell: bool


class PortfolioResponseData(BaseModel):
    clientId: str
    content: PortfolioContent
    equity: float
    balance: float
    positions: list = []
    currencies: list = []
    money: list = []


class PortfolioResponseModel(BaseModel):
    data: PortfolioResponseData



from typing import Union

from finam.base_client.base import BaseClient
from finam.models import ErrorBodyModel
from finam.order.model import (
    CreateOrderRequestModel,
    CreateOrderResponseModel,
    DelOrderModel,
    DelOrderResponseModel,
    DelStopOrderRequestModel,
)
from finam.order.model import DelStopOrderResponseModel as DelStopOrderResponse
from finam.order.model import (
    OrdersRequestModel,
    OrdersResponseModel,
    StopOrdersResponseModel,
)


class OrderClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._order_url = "/api/v1/orders"
        self._stop_order_url = "/api/v1/stop-orders"

    async def get_orders(self, params: OrdersRequestModel) -> Union[OrdersResponseModel, ErrorBodyModel]:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._order_url,
            params=params.dict(exclude_none=True)
        )
        if not ok:
            return ErrorBodyModel(**response)
        return OrdersResponseModel(**response)

    async def create_order(self, payload: CreateOrderRequestModel) -> Union[CreateOrderResponseModel, ErrorBodyModel]:
        response, ok = await self._exec_request(
            self.RequestMethod.POST,
            self._order_url,
            payload.dict(exclude_defaults=True)
        )
        if not ok:
            return ErrorBodyModel(**response)
        return CreateOrderResponseModel(**response)

    async def del_order(self, params: DelOrderModel) -> Union[DelOrderResponseModel, ErrorBodyModel]:
        response, ok = await self._exec_request(
            self.RequestMethod.DELETE,
            self._order_url,
            params=params.dict()
        )
        if not ok:
            return ErrorBodyModel(**response)
        return DelOrderResponseModel(**response)

    async def get_stop_orders(self, params: OrdersRequestModel) -> Union[StopOrdersResponseModel, ErrorBodyModel]:
        response, ok = await self._exec_request(
            self.RequestMethod.GET,
            self._stop_order_url,
            params.dict(exclude_none=True)
        )
        if not ok:
            return ErrorBodyModel(**response)
        return StopOrdersResponseModel(**response)

    async def del_stop_order(self, params: DelStopOrderRequestModel) -> Union[DelStopOrderResponse, ErrorBodyModel]:
        response, ok = await self._exec_request(
            self.RequestMethod.DELETE,
            self._stop_order_url,
            params.dict()
        )
        if not ok:
            return ErrorBodyModel(**response)
        return DelStopOrderResponse(**response)



from typing import Union

from finam.base_client.base import BaseClient
from finam.models import ErrorBodyModel
from finam.portfolio.model import PortfolioRequestModel, PortfolioResponseModel


class PortfolioClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._url = "/api/v1/portfolio"

    async def get_portfolio(self, params: PortfolioRequestModel) -> Union[PortfolioResponseModel, ErrorBodyModel]:
        p = {
            "clientId": params.clientId,
            "content.IncludeCurrencies": params.includeCurrencies,
            "content.IncludeMoney": params.includeMoney,
            "content.IncludePositions": params.includePositions,
            "content.IncludeMaxBuySell": params.includeMaxBuySell,
        }
        response, ok = await self._exec_request(self.RequestMethod.GET, self._url, params=p)
        if not ok:
            return ErrorBodyModel(**response)
        return PortfolioResponseModel(**response)



from typing import List

from pydantic import BaseModel


class Security(BaseModel):
    code: str
    board: str
    market: str
    decimals: float
    lotSize: float
    minStep: float
    currency: str
    instrumentCode: str
    shortName: str
    properties: float
    timeZoneName: str
    bpCost: float
    accruedInterest: float
    priceSign: str
    ticker: str
    lotDivider: float


class SecurityData(BaseModel):
    securities: List[Security]


class SecurityResponseModel(BaseModel):
    data: SecurityData



from setuptools import find_packages, setup

setup(
    name="finam-trade-api",
    maintainer="DBoyara",
    maintainer_email="boyarshin.den@yandex.ru",
    packages=find_packages(),
    version="0.2.4",
    install_requires=["aiohttp >= 3.8.3, < 4.0.0", "pydantic >= 1.10.2, < 2.0.0"],
    python_requires=">3.7.0, <4",
    license="GNU GPL v.3.0",
    description="",
    long_description="",
    url="https://github.com/DBoyara/FinamTradeApiPy",
)



from typing import Optional, Union

from finam.base_client.base import BaseClient
from finam.models import ErrorBodyModel
from finam.securities.model import Security, SecurityResponseModel


class SecurityClient(BaseClient):
    def __init__(self, token: str):
        super().__init__(token)
        self._url = "/api/v1/securities"

    async def get_data(self, code: Optional[str] = None) -> Union[SecurityResponseModel, Security, ErrorBodyModel]:
        response, ok = await self._exec_request(self.RequestMethod.GET, self._url)
        if not ok:
            return ErrorBodyModel(**response)
        s = SecurityResponseModel(**response)
        if code:
            for security in s.data.securities:
                if security.code.lower() != code.lower():
                    continue
                return security
        return s
